@@ -1,3 +1,4 @@
+import warnings
 from .grad_fn import *
 from typing import List, Tuple, Union
 import gc
@@ -119,35 +120,37 @@ class Node:
     def backward(self, gradients: GlobalGraph.np.ndarray = None):
         if self.grad_fn is None:
             raise ValueError('can not solve grad on %s' % self)
-        # 如果是标量，默认该标量梯度为1
-        if self.data.size == 1:
-            if self.grad is None:
-                self.grad = 1.
-            self.grad_fn(self)
-            if GlobalGraph.outputs is None:
-                GlobalGraph.outputs = self
-            if GlobalGraph.inputs is not None:
-                graph = GlobalGraph.build_graph()
-                GlobalGraph.inputs.retain_grad()
-                GlobalGraph.outputs.retain_grad()
-                for node in reversed(graph):
-                    if node.grad_fn is not None:
-                        node.grad_fn(node)
-                    if node.retain:
-                        continue
-                    del node.cache, node.data, node.grad, node.grad_fn, node.in_bounds, node.out_bounds, node.shape, \
-                        node.name, node.requires_grad
-                    del node
-                gc.collect()
-                # 这里反向传播计算完就把这个图销毁，在C++中就是释放内存,把GlobalGraph的inputs, outputs和graph的内存都释放掉，或者说重置参数
-                GlobalGraph.reset_graph()
-
-        # 如果不是标量，但是手动传入该tensor的梯度，也可以求导，但是一定要让传入的梯度shape和tensor的shape对上
-        elif gradients is not None:
+        if gradients is not None:
+            assert gradients.size == self.data.size and gradients.shape == self.data.shape
             self.grad = gradients
-            self.grad_fn(self)
         else:
-            raise ValueError('grad can be implicitly created only for scalar outputs')
+            # 如果是标量，默认该标量梯度为1
+            if self.data.size == 1:
+                if self.grad is None:
+                    self.grad = 1.
+            else:
+                raise ValueError('grad can be implicitly created only for scalar outputs')
+
+        self.grad_fn(self)
+        if GlobalGraph.outputs is None:
+            GlobalGraph.outputs = self
+        if GlobalGraph.inputs is not None:
+            graph = GlobalGraph.build_graph()
+            GlobalGraph.inputs.retain_grad()
+            GlobalGraph.outputs.retain_grad()
+            for node in reversed(graph):
+                if node.grad_fn is not None:
+                    node.grad_fn(node)
+                if node.retain:
+                    GlobalGraph.reset_node(node)
+                else:
+                    GlobalGraph.delete_node(node)
+            # 这里反向传播计算完就把这个图销毁，在C++中就是释放内存,把GlobalGraph的inputs, outputs和graph的内存都释放掉，或者说重置参数
+            GlobalGraph.reset_graph()
+            gc.collect()
+
+    def size(self, axis: int = None):
+        return self.data.shape if axis is None else self.data.shape[axis]
 
     def zero_grad(self):
         self.grad = np.zeros_like(self.data)
@@ -236,7 +239,7 @@ class Node:
 
         outputs = Variable(in_bounds=[self], data=self.data.reshape(*shapes))
         # 绑定反向求梯度的函数
-        outputs.grad_fn = AbsBackward
+        outputs.grad_fn = ViewBackward
         initialize_ops_grad(self)
         self.out_bounds.append(outputs)
         return outputs
@@ -274,13 +277,23 @@ class Node:
         self.out_bounds.append(outputs)
         return outputs
 
+    def max(self, axis=None):
+        outputs = Variable(in_bounds=[self], data=np.max(self.data, axis=axis))
+        return outputs
+
+    def argmax(self, axis=None):
+        outputs = Variable(in_bounds=[self], data=np.argmax(self.data, axis=axis))
+        return outputs
+
 
 class Variable(Node):
     def __init__(self, data: Union[GlobalGraph.np.ndarray, int, float], in_bounds: List = None,
                  out_bounds: Union[List, Tuple] = None,
-                 name: str = None, requires_grad: bool = True, dtype=np.float64):
+                 name: str = None, requires_grad: bool = True, dtype: str = 'float64'):
         # Variable初始化时必须提供data值，data值可能传入的是一个int或者float，我们需要把它包装成numpy矩阵（cpp中就包装成我们选的矩阵库类型）
-        data = np.asarray(data, dtype=dtype)  # 数据类型用float64吧，float32也行
+        dtype_dict = {'int': np.int, 'float': np.float, 'int8': np.int8, 'int16': np.int16, 'int32': np.int32,
+                      'int64': np.int64, 'float32': np.float32, 'float64': np.float64}
+        data = np.asarray(data, dtype=dtype_dict[dtype])  # 数据类型用float64吧，float32也行
         Node.__init__(self,
                       in_bounds=in_bounds,
                       out_bounds=out_bounds,
@@ -301,10 +314,12 @@ class Variable(Node):
 class Constant(Node):
     def __init__(self, data: Union[GlobalGraph.np.ndarray, int, float], in_bounds: List = None,
                  out_bounds: Union[List, Tuple] = None,
-                 name: str = None):
+                 name: str = None, dtype: str = 'float64'):
         # Constant初始化时必须提供data值，并且一旦初始化就不可修改，Constant因为值不需要修改，也就没必要计算梯度，默认require_grads为False
         # !!!!!!!!!!!!!!!!注意Constant里的这个data要是const类型的，总之就是Constant的data一旦初始化后没办法被修改
-        data = np.asarray(data, dtype=np.float64)
+        dtype_dict = {'int': np.int, 'float': np.float, 'int8': np.int8, 'int16': np.int16, 'int32': np.int32,
+                      'int64': np.int64, 'float32': np.float32, 'float64': np.float64}
+        data = np.asarray(data, dtype=dtype_dict[dtype])
         # 显示调用父类的初始化函数
         Node.__init__(self,
                       in_bounds=in_bounds,
@@ -314,6 +329,12 @@ class Constant(Node):
                       shape=data.shape,
                       requires_grad=False)
 
+    def __setattr__(self, key, value):
+        if key == 'data' and key in self.__dict__:
+            warnings.warn('Can not change the value of a Constant!')
+        else:
+            self.__dict__[key] = value
+
 
 class Layer:
     def __init__(self, in_bounds: List = None, out_bounds: List = None, input_shape: Union[List, Tuple] = None,
